MODULE ChrDemo;
(* Demo belonging to CoreFont(XtdLib package), (C) by Marco van de Voort 1996
   VGA/EGA only!!*)

FROM Str      IMPORT Length;
FROM CoreFont IMPORT GetSetChar,GetSetFont,BChar,FontP,BlowUpBitset8,
                      MirrorBitset8,Bit9,SetCharWidth8;

FROM ELib     IMPORT GetKey;
FROM Storage  IMPORT ALLOCATE,DEALLOCATE;
FROM Lib      IMPORT Move;

IMPORT Crt,IO;

TYPE SmallFnt=ARRAY[1..21] OF BChar;

(* Several special characters used, 8x16 bitmapped font *)
CONST ChrSet=SmallFnt(
      BChar(000H,000H,0E7H,0A5H,0E7H,000H,000H,008H,018H,038H,000H,000H,0C3H,0C3H,07EH,000H),
      BChar(000H,000H,0E7H,0A5H,0E7H,000H,000H,008H,018H,038H,000H,000H,07EH,0C3H,0C3H,000H),
      BChar(0AAH,0AAH,0FEH,000H,0EEH,0AAH,0EEH,000H,008H,018H,038H,000H,0C6H,0C6H,07CH,000H),
      BChar(003H,007H,0FFH,000H,00EH,00AH,00EH,000H,000H,001H,003H,000H,008H,007H,000H,000H),
      BChar(0C0H,0E0H,0FFH,000H,0E0H,0A0H,0E0H,000H,080H,080H,080H,010H,010H,0E0H,000H,000H),
      BChar(0E7H,042H,000H,0C3H,0A5H,0E7H,000H,008H,018H,038H,000H,000H,07EH,0FFH,081H,000H),
      BChar(000H,0FEH,082H,082H,082H,044H,028H,010H,038H,054H,0BAH,0FEH,0FEH,0FEH,000H,000H),
      BChar(24,0,66,0,0,024,165,24,60,102,66,66,66,102,60,0),
      BChar(24,126,255,231,231,255,255,255,255,255,191,255,255,255,255,255),
      BChar(24,24,24,24,24,24,24,24,24,24,126,24,24,24,60,24),
      BChar(8,42,28,127,27,42,8,8,8,8,8,8,8,8,8,0),
      BChar(0,0,102,60,24,24,24,60,60,126,126,126,60,24,0,0),
      BChar(255,171,213,171,213,171,213,171,213,171,213,171,213,171,213,171),
      BChar(24,60,102,102,102,60,24,24,24,24,120,120,24,120,120,0),
      BChar(9,64,4,33,0,136,2,32,1,136,0,66,0,8,64,18),
      BChar(119,119,119,0,238,238,238,0,119,119,119,0,238,238,238,0),
      BChar(0,0,0,0,0,0,0,192,70H,48,24,24,24,24,24,24),
      BChar(24,24,24,24,0CH,0EH,3,0,0,0,0,0,0,0,0,0),
      BChar(24,24,24,24,48,70H,192,0,0,0,0,0,0,0,0,0),
      BChar(0,0,0,0,0,0,0,3,0EH,0CH,24,24,24,24,24,24),
      BChar(0,0,0,0,0,3,0EH,19H,33H,54,54,54,54,54,54,54));

PROCEDURE WrBStr(S:ARRAY OF CHAR);

(* Write a string in a special way. A character becomes twice as big in
    both dimensions. When C is the ordinal value of the character,
    the big character looks like this (ASCII/CHR() codes)

        C       C+64

        C+128   C+196 *)

VAR A,X,Y:CARDINAL;

BEGIN
 IF Length(S)>0 THEN
  (* Save current cursor positions to use in calculations *)
  X:=Crt.WhereX();   Y:=Crt.WhereY();
  FOR A:=0 TO Length(S)-1 DO
   IF S[A] # CHR(32) THEN (* no space, then draw big character
                                as described above*)
    Crt.GotoXY(A*2,Y);
    IO.WrChar(S[A]);
    Crt.GotoXY(X+A*2+1,Y);
    IO.WrChar(CHR(ORD(S[A])+64));
    Crt.GotoXY(X+A*2,Y+1);
    IO.WrChar(CHR(ORD(S[A])+128));
    Crt.GotoXY(X+A*2+1,Y+1);
    IO.WrChar(CHR(ORD(S[A])+160));
   ELSE                   (* space? than draw four spaces, to keep the space
                              entirely blank*)
    Crt.GotoXY(A*2,Y);
    IO.WrChar(CHR(32));
    IO.WrChar(CHR(32));
    Crt.GotoXY(A*2,Y+1);
    IO.WrChar(CHR(32));
    IO.WrChar(CHR(32));
    END;
   END;
  END;
END WrBStr;

PROCEDURE BlowUpRange(Starting,Number:CARDINAL);
(* Blows up a range in the current font to twice its dimensions.
   Starting+Number<=64, since there's only room for 64 blown up characters*)

VAR A0,A1,A2,A3,A4 : BChar;
    A,B,T          : CARDINAL;

BEGIN
 FOR A:= 0 TO Number-1 DO
  GetSetChar(A+Starting,0,A0,FALSE);  (* Get a char from current biosfont *)
  FOR B:=0 TO 15 DO
   T:=CARDINAL(A0[B]);
   T:=BlowUpBitset8(BYTE(T));        (* Blow-up a row *)
   IF B<8 THEN                       (* First 8 rows in upper characters*)
    A1[B*2]:=BYTE(T AND 255);        (* right part blownup row *)
    A1[B*2+1]:=BYTE(T AND 255);      (* do that twice -> doubles lines *)
    A2[B*2]:=BYTE((T AND 0FF00H) >>8); (* left part blownup row *)
    A2[B*2+1]:=BYTE((T AND 0FF00H)>>8);
   ELSE
    A3[B*2-16]:=BYTE(T AND 255);        (* Same for lower 8 rows in lower *)
    A3[B*2-15]:=BYTE(T AND 255);        (* characters*)
    A4[B*2-16]:=BYTE((T AND 0FF00H)>>8);
    A4[B*2-15]:=BYTE((T AND 0FF00H)>>8);
    END;
   END;
  GetSetChar(A+Starting,0,A2,TRUE);    (* Write the 4 chars to the current *)
  GetSetChar(A+Starting+64,0,A1,TRUE); (* Bios font *)
  GetSetChar(A+Starting+128,0,A4,TRUE);
  GetSetChar(A+Starting+160,0,A3,TRUE);
  END;
END BlowUpRange;

VAR OldFnt,Fnt2         : FontP;
    A,B                 : CARDINAL;
    L                   : SmallFnt;

BEGIN
 Crt.ClrScr;            (* Clear screen *)
 SetCharWidth8;
 L:=ChrSet;             (* Make the constant a variable *)
 IO.WrStr("@ABCDEFGHIJKLMNOPQRSTUVWXYZ"); IO.WrLn;
 NEW(OldFnt);           (* Allocate space for font *)
 GetSetFont(255*256+0,0,OldFnt^,FALSE); (* save current font to OldFnt *)
 A:=GetKey();            (* Wait for a key*)
 FOR A:= 1 TO 21 DO      (* Activate special signs *)
  GetSetChar(64+A,0,L[A],TRUE); (* this could be done more efficiently by *)
  END;                     (* using GetSetFont(01441H,0,L,TRUE), but I
                              wanted to demonstrate GetSetChar *)
 A:=GetKey();
 GetSetFont(20*256+65,0,OldFnt^[65][0] ,TRUE);  (* restore font to normal from
                                                   saved font OldFnt *)
 Crt.ClrScr;
 IO.WrStr ("Small hey? Couldn't you read it ? [Press a key]"); A:=GetKey();
 GetSetFont(01441H,0,L,TRUE); (* Reactivate special signs *)
 Crt.ClrScr;
 WrBStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
 BlowUpRange(65,26);          (* Blowup 26 characters starting with CHR(65) *)
 A:=GetKey();
 GetSetFont(255*256,0,OldFnt^,TRUE); (* Restore standard font *)
 BlowUpRange(65,26);                 (* Blow up standard font *)
 A:=GetKey();
 GetSetFont(255*256,0,OldFnt^,TRUE); (* Restore standard font *)
 Crt.ClrScr;
 FOR A:=32 TO 255 DO
  IO.WrChar(CHR(A));
  END;
 NEW(Fnt2);                     (* Allocate space for Fnt2*)
 Move (OldFnt,Fnt2,256*16);     (* Copy standard font to Fnt2 *)
 FOR A:=0 TO 255 DO
  FOR B:=0 TO 15 DO
   Fnt2^[A][B]:=MirrorBitset8(Fnt2^[A][B]); (* mirror every byte of Fnt2 *)
   END;
  END;
 GetSetFont(0F000H,0,Fnt2^,TRUE);       (*  Activate Fnt2*)
 A:=GetKey();
 GetSetFont(255*256,0,OldFnt^,TRUE);    (* Restore standard font *)

 DISPOSE(Fnt2);                         (* Deallocate all used space*)
 DISPOSE(OldFnt);
END ChrDemo.