MODULE CrToLf;
(* Filter proggie, XtdLib package, (C) by Marco van de Voort 1997

Originally a lf to crlf filter, for 00_index.txt's picked of unix machines.

Since I'm more and more into unix/linux, I started using the original very
simple program more, and decided to expand it. In future, it may be integrated
into WordW, to form a complete textformatter.

Scans a textfile changing
- (CR,LF and CRLF) to (CR or LF or CRLF), configurable, what do you want,
   Cr or Lf or CrLf?
- Expanding hardtabs to a configurable number spaces and vice versa

Meant to be expanded in future, for heavier and more configurable filtering.
(HighAscii, country-shit (ae and a" to a-umlaut and vice versa)) or just
a simple hold all signs in a certain CHARSET. Simply because sometimes people
use high or low (0..31) ascii in textfiles to code colors, spacings etc etc.
See e.g. the helpfile source of TopSpeed 3.1x! If you hold CHR(2) and 3, you
can convert it to a normal (but unusable for helpcompiler!) textfile.
The number of spaces a hardtab stands for is a problem.
- Normal is 8, but the often used dos-editor q.exe has a tabsize dependant on
  the extension of the file.
- Some programs have real tabs, so a hardtab means go to next tabulator, and
  the first can be on 8, the second on 15 the third on 18 etc.
  This is not implemented yet.

8 is default now default here, if your file doesn't look good, try another
value.

All configurable items are hardcoded in constants in the main module.
I should make some command switches for this, and the original file should
be renamed to <file>.bak, and the new one should get the original file's name.
*)

IMPORT FIO;
FROM Lib IMPORT ParamStr,ParamCount;
FROM Str IMPORT CHARSET,Length,Append,Copy;
FROM ELib IMPORT ChangeExtension;

TYPE LineSepType=(CrLf,Lf,Cr);

CONST ToTab =TRUE; (* TRUE means spaces to tab, FALSE tab to spaces *)
      TabLength=8; (* hardtab=8 spaces *)
      LineSep=CrLf; (* What do I want? Cr, Lf or CrLf? The other two types are
                        converted to this type *)

(* These characters trigger action: (CR,LF,TAB,SPACE) *)
      Trigger=CHARSET{CHR(13),CHR(10),CHR(9),CHR(32)};


VAR F1,F2 :FIO.File;
    B1,B2:ARRAY[0..4095+FIO.BufferOverhead] OF BYTE;
    C    : CHAR; (* Current read characted *)
    S    : ARRAY[0..100] OF CHAR; (* A buffer for characters to save*)
    InBuf : BOOLEAN;

PROCEDURE Flush;
(* Flush the buffer when filled*)

BEGIN
 IF InBuf THEN
  IF S[0]=CHR(13) THEN (* CHR(10) never gets in "S", so we don't have to
                           test for it *)
   CASE LineSep OF     (* make sure that only the right type is flushed*)
    Lf : FIO.WrChar(F2,CHR(10));
 |  Cr : FIO.WrChar(F2,CHR(13));
 |  CrLf: FIO.WrLn(F2);
    END;
  ELSE
   FIO.WrStr(F2,S);
   END;
  S[0]:=CHR(0); InBuf:=FALSE;
  END;
END Flush;

VAR A : CARDINAL;
    S2,S3 : FIO.PathStr;

BEGIN
 A:=ParamCount();
 IF (A=0) THEN
  HALT;
  END;
 ParamStr(S,1);
 F1:=FIO.Open(S);   FIO.AssignBuffer(F1,B1);
 IF A=2 THEN
  ParamStr(S,2);
 ELSE
  Copy(S2,S);
  ChangeExtension(S,"TMP");
  END;
 F2:=FIO.Create(S); FIO.AssignBuffer(F2,B2);
 S[0]:=CHR(0); InBuf:=FALSE;
 LOOP
  C:=FIO.RdChar(F1);
  IF FIO.EOF THEN
   EXIT
   END;
  IF C IN Trigger THEN
   CASE C OF
   | CHR(13) : Flush;
               S[0]:=CHR(13); S[1]:=CHR(0);
               InBuf:=TRUE;
   | CHR(10) : IF S[0] # CHR(13) THEN
                Flush;
                END;
               (* Not imported if CHR(13) in buffer, as long as it's not
                  written*)
               S[0]:=CHR(0); InBuf:=FALSE;
               CASE LineSep OF
                Lf : FIO.WrChar(F2,CHR(10));
             |  Cr : FIO.WrChar(F2,CHR(13));
             |  CrLf: FIO.WrLn(F2);
                END;
   | CHR(9)  : Flush;
               IF ToTab THEN
                FIO.WrChar(F2,C);
               ELSE
                FIO.WrCharRep(F2," ",TabLength);
                END;
   | CHR(32) : IF S[0] # CHR(32) THEN
                Flush;
                END;
               Append(S,C); InBuf:=TRUE;
               IF Length(S)=(TabLength) THEN
                FIO.WrChar(F2,CHR(9));
                InBuf:=FALSE; S[0]:=CHR(0);
                C:="";
                END;
    END;
   ELSE
    Flush;
    FIO.WrChar(F2,C);
   END;
  END;
 FIO.Close(F1);
 FIO.Close(F2);
 IF A=1 THEN
  Copy(S3,S2);
  ChangeExtension(S3,"BAK");
  IF FIO.Exists(S3) THEN
   FIO.Erase(S3);
   END;
  FIO.Rename(S2,S3);
  FIO.Rename(S,S2);
  END;
END CrToLf.
