MODULE Dict;
(* Dict, demo for Crt(XtdLib package), (C) by Marco van de Voort 1996
   The entire program requires VGA.

   Dict is a abbreviation for Dictatuur(dutch for Dictatorship). It's a form
   of the game of Life.

   The reason that the particular version I tried to reproduce here was
   called Dictatorship, was that the original version of procedure DoStuff
   was different, and the screen it converted was not convergent, neither
   color took control, this was called Democracy. By a small change in the
   algoritm of DoStuff one  of the colors filled the screen at the end,
   like in a Dictatorship.

   There are 4 versions

   The difference between Dict and Dict2 is the handling of the borders of the
   screen, in Dict the border units do not have neighbours outside the screen,
   so these units do not have 8 neigbours but 5, or even 3 (in the corners).
   In Dict2 the border-units of the screen are fixed on the initial colors,
   which makes colorsurface 'stick' on the walls.

   Dict is also a demonstration of the virtual screen.

   Dict3 and Dict4 are graphical version of resp. Dict and Dict2, and therefore
   Dict3 and Dict4 have a higher resolution, but are much slower.
   I recommend to compile Dict3 and Dict4 with a coprocessor on.

   Watch out, the const WeirdMode is a modenumber that is adapter specific.
   It sets my Trident in 120x60 mode. Adapt to a value specific for your own
   adapter.

   *)


FROM Crt IMPORT Red,White,Black,Color,ClrScr,Detect;
FROM CoreCrt IMPORT Screen,ScrDat,ScrP,ChangeMode;
FROM Lib IMPORT RANDOMIZE,RANDOM,Intr,FarWordMove;
FROM IO  IMPORT RdKey,KeyPressed,WrStr,WrLn,WrLngCard;
FROM ELib IMPORT GetKey;
IMPORT Storage;

CONST WeirdMode=86;                       (* <------------ Watch out *)

CONST WhiteCrit    =3;
      RedCrit      =3;
      RandWhiteCrit=1;
      RandRedCrit  =1;
      RefR=1;

PROCEDURE DoGeneration;

VAR A,B,C,
    CWhite,CRed : CARDINAL;

PROCEDURE Process(Value:CARDINAL);

BEGIN
 IF Value=ORD(Red) THEN
  INC(CRed);
 ELSIF Value=ORD(White) THEN
  INC(CWhite);
  END;
END Process;

PROCEDURE DoStuff(CritW,CritR:CARDINAL);

BEGIN
 (* Count Neighbours *)
 CWhite:=0;
 CRed  :=0;
 IF A # 0 THEN
  IF B # 0 THEN
   Process(Screen^[(A-1)*CARDINAL(ScrDat.Width)+B-1] DIV 4096);
   END;
  Process(Screen^[(A-1)*CARDINAL(ScrDat.Width)+B] DIV 4096);
  IF B # (CARDINAL(ScrDat.Width)-1) THEN
   Process(Screen^[(A-1)*CARDINAL(ScrDat.Width)+B+1] DIV 4096);
   END;
  END;
 IF B#0 THEN
  Process(Screen^[(A)*CARDINAL(ScrDat.Width)+B-1] DIV 4096);
  END;
 IF B # (CARDINAL(ScrDat.Width)-1) THEN
  Process(Screen^[(A)*CARDINAL(ScrDat.Width)+B+1] DIV 4096);
  END;
 IF A # CARDINAL(ScrDat.Depth) THEN
  IF B#0 THEN
   Process(Screen^[(A+1)*CARDINAL(ScrDat.Width)+B-1] DIV 4096);
   END;
  Process(Screen^[(A+1)*CARDINAL(ScrDat.Width)+B] DIV 4096);
  IF B # (CARDINAL(ScrDat.Width)-1) THEN
   Process(Screen^[(A+1)*CARDINAL(ScrDat.Width)+B+1] DIV 4096);
   END;
  END;
 IF (CWhite>CritW) AND (CRed>CritR) THEN
  C:=RANDOM(2);    (* Overcrowded *)
  IF C=1 THEN
   Screen^[A*CARDINAL(ScrDat.Width)+B]:=32+ORD(Red  )*4096;
  ELSE
   Screen^[A*CARDINAL(ScrDat.Width)+B]:=32+ORD(White)*4096;
   END;
 ELSIF ((CWhite<CritW) AND (CRed<CritR)) THEN
  (* Do Nothing *)
 ELSE
  IF CWhite>CritW THEN
   Screen^[A*CARDINAL(ScrDat.Width)+B]:=32+ORD(White)*4096;
   END;
  IF CRed  >CritR THEN
   Screen^[A*CARDINAL(ScrDat.Width)+B]:=32+ORD(Red)*4096;
   END;
  END;
END DoStuff;

BEGIN
 FOR A:=0 TO CARDINAL(ScrDat.Depth) DO
  FOR B:=0 TO CARDINAL(ScrDat.Width)-1 DO
   IF (B=0) AND ((A>0) AND (A<CARDINAL(ScrDat.Depth)) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
   IF (B=CARDINAL(ScrDat.Width)-1) AND ((A>0) AND (A<CARDINAL(ScrDat.Depth)) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
   IF (B<CARDINAL(ScrDat.Width)-1) AND ((B>0) AND (A=CARDINAL(ScrDat.Depth)) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
   IF (B<CARDINAL(ScrDat.Width)-1) AND ((B>0) AND (A=0) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
    IF ((B>0) AND (B<CARDINAL(ScrDat.Width)-1)) AND ((A>0) AND (A<CARDINAL(ScrDat.Depth))) THEN
    DoStuff(WhiteCrit,RedCrit);
     END;
   END;
  END;
END DoGeneration;

VAR A,X,Y : CARDINAL;
    C     : Color;
    Re,Bl,Wh,Nr:LONGCARD;
    VScr  : ScrP;
    Fl    : BOOLEAN;

BEGIN
 Storage.FarAllocate(VScr,16384);
 ChangeMode(WeirdMode);
 Detect;
 Screen:=VScr;
 RANDOMIZE;
 FOR Y:=0 TO CARDINAL(ScrDat.Depth) DO
  FOR X:= 0 TO CARDINAL(ScrDat.Width)-1 DO
   A:=RANDOM(3);
   Fl:=ODD(Y);
   IF X>((CARDINAL(ScrDat.Width) DIV 2)-1) THEN
    Fl:=NOT Fl;
    END;
   IF Fl THEN
    IF A=1 THEN
     C:=Red;
    ELSE
     C:=White;
     END;
   ELSE
    IF A=1 THEN
     C:=White;
    ELSE
     C:=Red;
     END;
    END;
   Screen^[X+CARDINAL(ScrDat.Width)*Y]:=32+ORD(C)*4096;
   END;
  END;
 A:=0;    Nr:=0;
 Wh:=0;   Bl:=0;
 Re:=0;
 FarWordMove(FarADR(Screen^[0]),[0B800H:0],8192);
 LOOP
  IF KeyPressed() THEN
   A:=CARDINAL(RdKey());
   EXIT;
   END;
  DoGeneration;
  INC(Nr);
  IF (Nr MOD RefR)=0 THEN
   FarWordMove(FarADR(VScr^[0]),[0B800H:0],8192);
   END;
  END;
 FOR Y:=0 TO CARDINAL(ScrDat.Depth) DO
  FOR X:= 0 TO CARDINAL(ScrDat.Width)-1 DO
   A:=Screen^[X+CARDINAL(ScrDat.Width)*Y] DIV 4096;
   IF A=ORD(Red) THEN
    INC(Re);
   ELSIF A=ORD(White) THEN
    INC(Wh);
   ELSIF A=ORD(Black) THEN
    INC(Bl);
    END;
   END;
  END;
 Screen:=[0B800H:0];
 ClrScr;
 WrStr("Nr of tests : ");
 WrLngCard(Nr,8);
 WrLn;
 WrStr("White parts : ");
 WrLngCard(Wh,8);
 WrLn;
 WrStr("Red   parts : ");
 WrLngCard(Re,8);
 WrLn;
 WrStr("Black parts : ");
 WrLngCard(Bl,8);
 WrLn;
 A:=GetKey();
 ChangeMode(3);
END Dict.
