MODULE Indexer;

(* Another demonstration of module DirTree, XtdLib package,
                              (C) by Marco van de Voort 1996

   Demo of the 'include directories too' feature. The program is even usefull,
   it tries to create an index per directorysystem (files.bbs,
   00_index.txt, and maybe in the future a HTML system) and a main index
   (main HTML in the future). Goal is an index-system like on Walnutt Creek
   CDs, I've purchased a WC Simtel CD, and it turned out to be the best indexed
   CD I've ever seen.

   Since I'm a fervent NDOS/4DOS-user, and have described 99% of my files, I
   start with descript.ions. If you however want to start with files.bbs or
   00_index.txt that's also possible with some minor changes. (files.bbs is
   mere changing some filenames, 00_index has also directories in it, a little
   bit more difficult, but can be fixed in an hour or so)

   Assumes the parameter-directory is a starting point for a future CD,
    browses through the directories, reading descript.ion, generating
    files.bbs and a general index in the parameter-directory

   This initial version works, but to obtain a nice output :
    - Describe all files, only described files get in the list.
       Described directories are ignored.
    - Run a directory sorter (like nortons disksort) to get the directories
      alphabetically sorted. If you don't own Norton, you could run a disk
      defragger, DOS 6.x has one included. Most disk-defraggers have file-sort
      directories included.
    - This version is not very robust, remove garbadge from descript.ion's
       when necessary. QPV comments are already deleted automatically.

For a nice product some things should be improved :

- Support for fileattributes, scan only archive/ not hidden etc files.
- A setupfile, where to read descriptions from, and what to create.
- HTML and 00_index.txt support, problem is that directory structures exist
   in these files, so the simple procedure executed here is not enough.
- Combining indexes. e.g. start with FILES.BBS, en add all nonmatches from
  descript.ion.
- Get all files, with a certain pattern (Wildcard(s)), and put them in a list
   with no description, and resort.

*)


FROM Lib     IMPORT ParamCount,ParamStr,ExecCmd,Move,Fill;
FROM ELib    IMPORT ExpPath,KillChar;
FROM Str     IMPORT Concat,Copy,Append,ItemS,Delete,Length,CHARSET,Compare,
                    Prepend,CharPos;
FROM Storage IMPORT ALLOCATE,DEALLOCATE;

IMPORT IO,FIO,Crt,DirTree;

TYPE DirEPoint  =    POINTER TO DirERec;
     DirERec    =    RECORD
                      Next  : DirEPoint;
                      DirE  : FIO.DirEntry;
                      Descr : ARRAY[0..127] OF CHAR;
                      END;

VAR MainDir : ARRAY[0..127] OF CHAR;
    MainF   : FIO.File;

PROCEDURE SortDescrTree(Root:DirEPoint;Nr : CARDINAL);
(* A bubble sorter I'm afraid, since I never figured a smarter way to
    sort a single linked tree. *)

VAR A           : CARDINAL;
    Cur,Cur2    : DirEPoint;
    Rec         : DirERec;

BEGIN
 FOR A:=1 TO Nr DO
  IF Root^.Next # NIL THEN
   Cur:=Root;
   LOOP
    Cur2:=Cur^.Next;
    IF Cur2^.Next= NIL THEN
     EXIT
     END;
    IF Compare(Cur^.DirE.Name,Cur2^.DirE.Name)>0 THEN
     Rec:=Cur^;
     Copy(Cur^.Descr,Cur2^.Descr);
     Cur^.DirE:=Cur2^.DirE;
     Copy(Cur2^.Descr,Rec.Descr);
     Cur2^.DirE:=Rec.DirE;
     END;
    Cur:=Cur^.Next;
    IF Cur^.Next=NIL THEN
     EXIT
     END;
    END;
   END;
  END;
END SortDescrTree;

PROCEDURE ShowRecord(Dir:FIO.PathStr;Fil:FIO.DirEntry);

VAR S,S2        : ARRAY[0..127] OF CHAR;
    TempF       : FIO.File;
    C           : CARDINAL;
    BBSF        : FIO.File;
    Root, Cur   : DirEPoint;
    A,Descrs      : CARDINAL;
    Fl          : BOOLEAN;

CONST DDe="\descript.ion";

BEGIN
 IO.WrStr(Dir); IO.WrLn;
 Copy(S,"attrib -rhs ");
 Append(S,Dir);
 Append(S,DDe);
 Append(S," >nul");
 C:=ExecCmd(S);
 Copy(S2,Dir);
 Append(S2,DDe);
 IF FIO.Exists(S2) THEN;
  FIO.WrLn(MainF); FIO.WrLn(MainF);
  FIO.WrStr(MainF,Dir); FIO.WrLn(MainF);
  FIO.WrLn(MainF);
  Descrs:=0;
  NEW(Root);
  Cur:=Root;
  Cur^.Next:=NIL;
  TempF:=FIO.Open(S2);
  Copy(S2,Dir);
  Append(S2,"files.bbs");
  BBSF:=FIO.Create(S2);
  LOOP
   FIO.RdStr(TempF,S);
   IF FIO.EOF THEN
    EXIT
    END;
   C:=CharPos(S,CHR(4));
   IF C # MAX(CARDINAL) THEN
    Delete(S,C,Length(S)-C+1);
    END;
   FOR A:=0 TO Length(S)-1 DO
    IF S[A]=CHR(9) THEN
     S[A]:=" ";
     END;
    END;
   ItemS(S2,S," "+CHR(9),0);
   Delete(S,0,Length(S2));
   Prepend(S2,Dir);
   Fl:=FIO.ReadFirstEntry(S2,FIO.FileAttr{FIO.hidden,FIO.readonly,FIO.directory,FIO.archive,FIO.system},Cur^.DirE);
   KillChar(S,CHARSET{" "});
   Copy(Cur^.Descr,S);
   INC(Descrs);
   NEW(Cur^.Next);
   Cur:=Cur^.Next;
   Fill(ADR(Cur^),SIZE(DirERec),64);
   Cur^.Next:=NIL;
   END;
  SortDescrTree(Root,Descrs);
  Cur:=Root;
  LOOP
   IF NOT (FIO.directory IN Cur^.DirE.attr) AND NOT (FIO.hidden IN Cur^.DirE.attr) THEN
    FIO.WrStrAdj(MainF,Cur^.DirE.Name,-13);
    FIO.WrLngCard(MainF,Cur^.DirE.size,-10);
    FIO.WrStr(MainF," ");
    FIO.WrStr(MainF,Cur^.Descr);
    FIO.WrLn(MainF);
    FIO.WrStrAdj(BBSF,Cur^.DirE.Name,-13);
    FIO.WrStr(BBSF," ");
    FIO.WrStr(BBSF,Cur^.Descr);
    FIO.WrLn(BBSF);
    END;
   IF Cur^.Next=NIL THEN
    EXIT
    END;
   Cur:=Cur^.Next;
   IF Cur^.Next=NIL THEN
    EXIT
    END;
   END;
  FIO.Close(TempF);
  FIO.Close(BBSF);
  END;
END ShowRecord;

CONST CR=CHR(13)+CHR(10);
      Usage=
 "Creates indexes and Files.bbs from descript.ion, recursing directories."+CR+
 "Usage : Indexer <Starting-Directory> "+CR+
 "      E.g. Indexer c:..\source"+CR+CR;

VAR Param1        : ARRAY[0..127] OF CHAR; (* Size of string is important,
                                               see notes about ExpPath in
                                               ELib.DEF or in the Help *)
    Fl            : BOOLEAN;

BEGIN
 Crt.TextColor(Crt.Yellow);
 IO.WrStr("Indexer 0.0001, (C) Marco van de Voort, XtdLib package");
 Crt.TextColor(Crt.LightGray);
 IO.WrLn; IO.WrLn;

 IF ParamCount() <1 THEN
   IO.WrStr(Usage);
   HALT;
   END;
 ParamStr(Param1,1);
 Fl:=ExpPath(Param1); (* Simplify the path, ..\ .\ etc are expanded,
                         makes scan a lot more robust, ExpPath has
                         however some theoretical drawbacks, see ELib.DEF.*)
 IF NOT FIO.Exists(Param1) THEN
  Crt.TextColor(Crt.LightRed);
  IO.WrStr("Error : Path doesn't exist."); IO.WrLn;
  HALT
  END;
 Copy(MainDir,Param1);
 IF MainDir[Length(MainDir)-1] #"\" THEN
  Append(MainDir,"\");
  END;
 Append(MainDir,"index.txt");
 MainF:=FIO.Create(MainDir);
 DirTree.DirsToo:=TRUE;
 DirTree.FileScan(Param1,".",ShowRecord);
 Crt.CrtOff;
 FIO.Close(MainF);
END Indexer.