MODULE TreeDemo;
(* A demonstration of module DirTree, XtdLib package,
                              (C) by Marco van de Voort 1996

*)

FROM Str     IMPORT Copy,Append;
FROM DirTree IMPORT BuildTree,DirTreePoint,FilePtr,SearchForFiles;
FROM Storage IMPORT ALLOCATE,DEALLOCATE;
FROM Lib     IMPORT ParamCount,ParamStr;
FROM ELib    IMPORT ExpPath;

IMPORT IO,FIO,Crt,DirTree;

PROCEDURE DisplayDir(DirPtr:DirTreePoint);

BEGIN
 IO.WrStr(DirPtr^.Name); IO.WrLn;
 IF DirPtr^.SubDirs # NIL THEN
  DisplayDir(DirPtr^.SubDirs);
  END;
 IF DirPtr^.NextDir # NIL THEN
  DisplayDir(DirPtr^.NextDir);
  END;
END DisplayDir;

PROCEDURE DisplayDirAndFiles(DirPtr:DirTreePoint);

PROCEDURE DisplayFile(Fil:FilePtr);

BEGIN
 IO.WrStr(DirPtr^.Name);
 IO.WrStr(Fil^.DirE.Name); IO.WrLn;
 IF Fil^.Next # NIL THEN
  DisplayFile(Fil^.Next);
  END;
END DisplayFile;

BEGIN
 IF DirPtr^.Files # NIL THEN
(*  Copy(DirName,DirPtr^.Name); *)
  DisplayFile(DirPtr^.Files);
  END;
 IF DirPtr^.SubDirs # NIL THEN
  DisplayDirAndFiles(DirPtr^.SubDirs);
  END;
 IF DirPtr^.NextDir # NIL THEN
  DisplayDirAndFiles(DirPtr^.NextDir);
  END;
END DisplayDirAndFiles;

PROCEDURE Usage;

BEGIN
 IO.WrStr("Searches a directory and below for matching files.");
 IO.WrLn;
 IO.WrStr("By building a tree-structure in memory");
 IO.WrLn;
 IO.WrStr("Usage : TreeDemo <Directory> <FileSpec>"); IO.WrLn;
 IO.WrStr("      E.g. FileFind c:..\source *.pkt");
 IO.WrLn; IO.WrLn;
END Usage;


VAR Root : DirTreePoint;
    Param1,Param2 : ARRAY[0..127] OF CHAR; (* Size string is important,
                                               see notes about ExpPath in
                                               ELib.DEF or in the Help *)
    Fl            : BOOLEAN;
BEGIN
 Crt.TextColor(Crt.Yellow);
 IO.WrStr("FileFinder 0.0001, (C) Marco van de Voort, XtdLib package");
 Crt.TextColor(Crt.LightGray);
 IO.WrLn; IO.WrLn;

 IF ParamCount() <2 THEN
   Usage;
   HALT;
   END;
 ParamStr(Param1,1); ParamStr(Param2,2);
 IF NOT FIO.Exists(Param1) THEN
  Crt.TextColor(Crt.LightRed);
  IO.WrStr("Error : Path doesn't exist."); IO.WrLn;
  HALT
  END;
 DirTree.DirsToo:=TRUE;
 Fl:=ExpPath(Param1); (* Simplify the path, ..\ .\ etc are expanded,
                         makes scan a lot more robust, ExpPath has
                         however some theoretical drawbacks, see ELib.DEF.*)
 Crt.CrtOn;
 Root:=BuildTree(Param1);                (* Build Directory tree in memory
                                            (SO ONLY DIRECTORIES!) *)
 SearchForFiles(Root,Param2);          (* add files selectively ( *.mod) to
                                           the tree*)
 Crt.CrtOff; (* Next output should be pipable, normal output*)
 DisplayDirAndFiles(Root);              (* Display found files with complete
                                            path*)
 IO.WrCharRep("-",78);
 IO.WrLn;
 IO.WrCard(DirTree.FoundCount,25); IO.WrStr(" files, ");
 IO.WrLngCard(DirTree.TotalBytes,12); IO.WrStr(" bytes in ");
 IO.WrCard(DirTree.FoundDirs,0); IO.WrStr(" directories");
 IO.WrLn; IO.WrLn;

END TreeDemo.