MODULE WordW;
(* WordW, Demo for nothing special (XTDLIB package) (C) Marco van de Voort 1996

 Formats textdocuments as follows:

 tttttttt
 tttttt
 ttttttttttt
 tttttt t ttt t tttt t ttttt tttt ttt ttttt ttttttt ttttt tttttt tttttt

ttttttttttttt

becomes:

tttttttt tttttt ttttttttttt  tttttt t ttt t tttt t ttttt tttt ttt ttttt ttttttt
ttttt tttttt tttttt

ttttttttttttt


I once needed this to make certain textfiles printable which were formatted
to a width of 25 characters.

The constant 'B' in the mainmodule defines the length of the formatted line+2.

It works quite nicely for normal textfiles, except :
- No support for hardtabs( not that difficult to implement, replace all
   09H characters by a space right after the FIO.RdStr, and let the wrapping
   do the rest) or use CrToLf.MOD on the file first.
- 2 linefeeds at the end of the file, one can easily be avoided, one not.
- No support for "!." etc, so !. remains !. and get on the next line
   separately.

Running twice kills some rare combinations of special characters and
    spaces, in theory running twice isn't enough, but practically it is. This
    is not necessary anyway for normal textfiles, only for files with escape
    like characters in it (meaning colors, spacing etc).
*)

IMPORT FIO;
FROM Str IMPORT Length,Delete,CharPos,Insert,CHARSET,Copy,Append,StrToCard;
FROM Storage IMPORT ALLOCATE;
FROM Lib IMPORT ParamCount,ParamStr;
FROM ELib IMPORT KillChar,KillBChar;
FROM IO IMPORT WrStr,WrLn;

TYPE StringT = ARRAY[0..500] OF CHAR;
     PString = POINTER TO StringT;
     BT      = ARRAY[0..4095] OF BYTE;
     BufP    = POINTER TO BT;

  CONST
    Ch= CHARSET{012C," "};
    TempName="TMP.TMP";

PROCEDURE Wrap(VAR St: PString; MaxLen: CARDINAL;
                VAR StrB:PString);
(* returns a String of no more than maxLength(St^) Characters With the last
   Character being the last space beFore maxLength(St^). On return st now has
   the remaining Characters left after the wrapping.                  *)

  VAR
    Len      : CARDINAL;
    X,
    OldLen,
    NewLen   : CARDINAL;

  BEGIN  (* Wrap *)
    Copy(StrB^,St^);
    IF Length(St^) <= MaxLen THEN  (* no wrapping required *)
      St^:="";
     ELSE
      OldLen := Length(StrB^);                (* save the length of the original String *)
      StrB^[MaxLen-1]:=CHR(0);                        (* set length to maximum *)
      REPEAT                     (* find last space in st beFore or at maxlen *)
       Delete(StrB^,Length(StrB^)-1,1);
      UNTIL (StrB^[Length(StrB^)-1] = " ") OR (Length(StrB^) = 0)   ;
      IF Length(St^) = 0 THEN                   (* no spaces in st, so chop at maxLen *)
       StrB^[MaxLen]:=CHR(0);
       END;
      NewLen :=  Length(StrB^);          (* save the Length of the newly wrapped String *)
      Delete(St^,0,NewLen);              (* getting rid of the wrapped portion *)
      END;
END Wrap; (* Wrap *)

VAR  S,S1  : PString;
     P1,P2 : ARRAY[0..100] OF CHAR;
     F1,F2 : FIO.File;
     B1,B2 : BufP;
     End   : BOOLEAN;
     B     : CARDINAL;
     D     : FIO.DirEntry;
     N,L   : CARDINAL;

PROCEDURE Flush(A:CARDINAL;Size:CARDINAL);

BEGIN
 WHILE Length(S^) > A DO
  Wrap(S,Size,S1);
  FIO.WrStr(F2,S1^);
  FIO.WrLn(F2);
  END;
END Flush;

BEGIN
 IF ParamCount()<1 THEN
  HALT;
  END;
 B:=82;
 ParamStr(P1,1);
 End:=FIO.ReadFirstEntry(P1,FIO.FileAttr{},D);
 ALLOCATE(B1,SIZE(BT));
 ALLOCATE(B2,SIZE(BT));
 ALLOCATE(S,SIZE(StringT));
 ALLOCATE(S1,SIZE(StringT));
 IF End THEN
  REPEAT
      F1:=FIO.Open(D.Name);
      F2:=FIO.Create(TempName);
      WrStr(D.Name); WrLn;
      FIO.AssignBuffer(F1,B1^);
      FIO.AssignBuffer(F2,B2^);
      FIO.RdStr(F1,S^);
      IF NOT FIO.EOF THEN
       LOOP
        Flush(B,B);
        FIO.RdStr(F1,P1);
        KillChar(P1,Ch);
        KillBChar(P1,Ch);
        N:=0;
        LOOP
         IF N>=(Length(P1)-1) THEN
          EXIT
          END;
         IF (P1[N]=CHR(32)) AND (P1[N+1]=CHR(32)) THEN
          Delete(P1,N,1);
          END;
         INC(N);
         END;
        IF Length(P1)=0 THEN
         Flush(0,B);
         FIO.WrLn(F2);
         END;
        IF Length(S^) >0 THEN
         IF (S^[Length(S^)-1] # "-") THEN
          IF (P1[0] #" ") THEN
           Append(S^," ");
           END;
         ELSE
          IF Length(S^)>1 THEN
           Delete(S^,Length(S^)-1,1);
           END;
          END;
         END;
        Append(S^,P1);
        IF FIO.EOF THEN
         Flush(0,B);
         EXIT
         END;
        END;
       ELSE
        FIO.WrStr(F2,S^);
        END;
      FIO.Close(F1);
      FIO.Close(F2);
      FIO.Erase(D.Name);
      FIO.Rename(TempName,D.Name);
   UNTIL NOT FIO.ReadNextEntry(D);

  IF FIO.Exists(TempName) THEN
   FIO.Erase(TempName);
   END;
  END; (* IF *)
END WordW.