IMPLEMENTATION MODULE CRC16;

(* Modula-2 CRC-16 calculation.

   written by : Andreas Reinicke, 2:284/402@fidonet.org
   version    : 1.00 (January 7, 1994)
*)

TYPE
   CalcBlock = RECORD
      CASE : CARDINAL OF
      | 0 : NewChar : BYTE;
            OldCRC  : CARDINAL;
      | 1 : TempCRC : BITSET;
            Index   : SHORTCARD;
      | 2 : NewCRC  : CARDINAL;
            dummy   : BYTE;
      END;
   END;

VAR
   CRCtab: ARRAY [0..255] OF BITSET;

PROCEDURE CreateCRC16Table( Poly : BITSET );
VAR
   I, J : CARDINAL;
   Temp : CARDINAL;
BEGIN
   FOR I := 0 TO 255 DO
      Temp := I*256;
      FOR J := 1 TO 8 DO
         IF Temp > 07FFFH THEN
            Temp := CARDINAL(BITSET((Temp MOD 08000H)*2)/Poly);
         ELSE
            Temp := Temp*2;
         END;
      END;
      CRCtab[I] := BITSET(Temp);
   END;
END CreateCRC16Table;

PROCEDURE CalcCRC( crc     : CARDINAL;
                   BufAdr  : SYSTEM.ADDRESS;
                   BufSize : CARDINAL ) : CARDINAL;
VAR
   segm, offs, I : CARDINAL;
BEGIN
   segm := SYSTEM.Seg( BufAdr^ );
   offs := SYSTEM.Ofs( BufAdr^ );
   FOR I := 0 TO BufSize-1 DO
      AddCRC( crc, BYTE( [segm:offs+I]^ ) );
   END;
   RETURN crc;
END CalcCRC;

PROCEDURE AddCRC( VAR crc : CARDINAL; ch : SYSTEM.BYTE );
VAR
   Calc : CalcBlock;
BEGIN
   Calc.OldCRC  := crc;
   Calc.NewChar := ch;
   Calc.TempCRC := Calc.TempCRC / CRCtab[CARDINAL(Calc.Index)];
   crc := Calc.NewCRC;
END AddCRC;

END CRC16.
